#include <gui/window.h>
#include <gui/rect.h>
#include <gui/textview.h>
#include <util/application.h>
#include <util/message.h>
#include <util/string.h>
#include <util/locker.h>

#include <webview.h>

#include <ChromeClientSyllable.h>
#include <FrameView.h>
#include <FrameLoadRequest.h>
#include <PageCache.h>
#include <Cache.h>
#include <SharedTimer.h>

using namespace os;

class BrowserChromeClient;

class TestWindow : public Window
{
	public:
		TestWindow( const Rect & cFrame );
		~TestWindow();

		void HandleMessage( Message * pcMsg );
		bool OkToQuit( void );

		void OpenURL( String cURL );

	private:
		BrowserChromeClient *m_pcChromeClient;
		WebView *m_pcWebView;
		TextView *m_pcTextView;
};
#define ID_URL_CHANGED	(ID_NEW_WINDOW + 1)

class BrowserChromeClient : public WebCore::ChromeClientSyllable
{
	public:
		BrowserChromeClient( TestWindow* pcWindow )
		{
			m_pcWindow = pcWindow;
		}
		void setWindowRect(const WebCore::FloatRect& cRect)
		{
			m_pcWindow->SetFrame( cRect );
		}
		WebCore::FloatRect windowRect()
		{
			return m_pcWindow->GetFrame();
		}
		WebCore::FloatRect pageRect()
		{
			return m_pcWindow->GetFrame();
		}
		float scaleFactor()
		{
			return 1;
		}
		void focus()
		{
			m_pcWindow->MakeFocus();
		}
		void unfocus()
		{
			m_pcWindow->MakeFocus( false );
		}
		void show()
		{
		}
		virtual WebCore::Page* createWindow(WebCore::Frame*, const WebCore::FrameLoadRequest& cRequest, const WebCore::WindowFeatures& )
		{
#if 0
			WebView *pcWebView = new WebView( m_pcWindow->GetBounds(), "webview", this, CF_FOLLOW_ALL );
			pcWebView->OpenURL( cRequest.resourceRequest() );

			return( pcWebView->GetWebCoreFrame()->page() );
#else
			return NULL;
#endif
		}
		void setStatusbarText(const WebCore::String& cString)
		{
		}
	private:
		TestWindow* m_pcWindow;
};

/* The global mutex is supplied by WebCore */
extern Locker g_cGlobalMutex;

TestWindow::TestWindow( const Rect &cFrame ) : Window( cFrame, "webview_wnd", "WebView Test Shell" )
{
	Rect cTextBounds, cWebBounds;

	cTextBounds = cWebBounds = GetBounds();

	m_pcChromeClient = new BrowserChromeClient( this );

	/* DANGER WILL ROBINSON! See the note in the TestApp constructor about this mutex! */
	g_cGlobalMutex.Lock();
	SetMutex( &g_cGlobalMutex );

	m_pcTextView = new TextView( Rect(), "textview", "", CF_FOLLOW_LEFT | CF_FOLLOW_RIGHT | CF_FOLLOW_TOP );
	m_pcTextView->SetMultiLine( false );
	m_pcTextView->SetEventMask( TextView::EI_ENTER_PRESSED );
	m_pcTextView->SetMessage( new Message( ID_URL_CHANGED ) );

	cTextBounds.bottom = m_pcTextView->GetPreferredSize(true).y;
	m_pcTextView->SetFrame( cTextBounds );
	AddChild( m_pcTextView );

	cWebBounds.top = cTextBounds.bottom + 1.0f;

	m_pcWebView = new WebView( cWebBounds, "webview", m_pcChromeClient, CF_FOLLOW_ALL );
	AddChild( m_pcWebView );

	View *pcView = m_pcWebView->GetWebCoreFrame()->view()->syllableWidget();
	pcView->SetResizeMask( CF_FOLLOW_ALL );
	pcView->SetFrame( cWebBounds );
	pcView->MakeFocus();
}

TestWindow::~TestWindow()
{
	RemoveChild( m_pcTextView );
	RemoveChild( m_pcWebView );
	delete m_pcChromeClient;
}

void TestWindow::HandleMessage( Message * pcMsg )
{
	switch( pcMsg->GetCode() )
	{
		case ID_SET_STATUS_BAR_TEXT:
		{
			printf( "ID_SET_STATUS_BAR_TEXT\n" );
			break;
		}
		case ID_SET_TITLE:
		{
			printf( "ID_SET_TITLE\n" );
			break;
		}
		case ID_SAVE_LINK:
		{
			printf( "ID_SAVE_LINK\n" );
			break;
		}
		case ID_URL_OPENED:
		{
			printf( "ID_URL_OPENED\n" );
			break;
		}
		case ID_LOAD_FINISHED:
		{
			printf( "ID_LOAD_FINISHED\n" );
			break;
		}
		case ID_ACTIVATE_URLEDIT:
		{
			printf( "ID_ACTIVATE_URLEDIT\n" );
			break;
		}
		case ID_RELOAD:
		{
			printf( "ID_RELOAD\n" );
			m_pcWebView->Reload();
			break;
		}
		case ID_COPY:
		{
			printf( "ID_COPY\n" );
			break;
		}
		case ID_FIND:
		{
			printf( "ID_FIND\n" );
			break;
		}
		case ID_REPEAT_FIND:
		{
			printf( "ID_REPEAT_FIND\n" );
			break;
		}
		case ID_NEW_WINDOW:
		{
			printf( "ID_NEW_WINDOW\n" );
			break;
		}
		case ID_URL_CHANGED:
		{
			uint32 nEvents = 0;
			pcMsg->FindInt( "events", &nEvents );
			if( nEvents & os::TextView::EI_ENTER_PRESSED )
			{
				String cURL = m_pcTextView->GetBuffer()[0];
				OpenURL( cURL );
			}
			break;
		}
		default:
		{
			printf( "Unknown message with code #%d\n", pcMsg->GetCode() );
			Window::HandleMessage( pcMsg );
			break;
		}
	}
}

bool TestWindow::OkToQuit( void )
{
	Application::GetInstance()->PostMessage( M_TERMINATE );
	return true;
}

void TestWindow::OpenURL( String cURL )
{
	WebCore::ResourceRequest cRequest;
	cRequest.setURL( WebCore::KURL( cURL.c_str() ) );

	m_pcWebView->OpenURL( cRequest );

	m_pcTextView->Set( cURL.c_str(), false );
}

class TestApp : public Application
{
	public:
		TestApp( const char *pzName, String cURL );
		~TestApp();

	private:
		TestWindow *m_pcWindow;
};

TestApp::TestApp( const char *pzName, String cURL ) : Application( pzName )
{
	m_pcWindow = new TestWindow( Rect( 0, 0, 600, 400 ) );
	m_pcWindow->CenterInScreen();

	/* DANGER WILL ROBINSON! The renderer MUST be started and running before we
	   call Window::Show(), because everything shares the same mutex and it is
	   VERY VERY PICKY about the order in which the locks are taken. If Show() is
	   called before the renderer has started it will lead to a deadlock on
	   g_cGlobalMutex. THIS IS A VERY BAD DESIGN, and I shall try to find a
	   sensible fix.

	   Yes, it did take me quite some time before I realised this was the case. */

	m_pcWindow->OpenURL( cURL );

	m_pcWindow->Show();
	m_pcWindow->MakeFocus();
}

TestApp::~TestApp()
{
	g_cGlobalMutex.Lock();

	WebCore::pageCache()->releaseAutoreleasedPagesNow();
	(WebCore::cache())->setDisabled(true);
	WebCore::stopSharedTimer();

	g_cGlobalMutex.Unlock();
}

int main( int argc, char *argv[] )
{
	String cURL = "";
	if( argc > 1 )
		cURL = argv[1];

	TestApp *pcApp = new TestApp( "webview_shell", cURL );
	pcApp->Run();

	return 0;
}

