/*  libwebview.so - Web rendering library for Syllable
 *  Copyright (C) 2008 Syllable Team
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

#include <gui/window.h>
#include <util/message.h>
#include <util/locker.h>

#include <webview.h>
#include <webframeview.h>
#include <frameloaderclient.h>

#include <ContextMenuClientSyllable.h>
#include <DragClientSyllable.h>
#include <FrameLoaderClientSyllable.h>
#include <EditorClientSyllable.h>
#include <InspectorClientSyllable.h>

#include <Frame.h>
#include <FrameLoader.h>

#include <WindowFeatures.h>
#include <CachedPage.h>
#include <Settings.h>
#include <FocusController.h>

#include <HitTestResult.h>
#include <CString.h>

#include <page/Page.h>
#include <page/EventHandler.h>

using namespace os;

/* WebCore::ContextMenuClientSyllable implementation */
class WebViewContextMenuClient : public WebCore::ContextMenuClientSyllable
{
	public:
		WebViewContextMenuClient( WebView* pcView )
		{
			m_pcView = pcView;
		}

		void downloadURL(const WebCore::KURL& url)
		{
/* XXXKV: These messages should target WebView instead, which sould call a callback within itself.
   The client can overload WebView to get those callback events */
#if 1
			printf( "downloadURL %s\n", url.string().utf8().data() );

			if( m_pcView->GetWindow() != NULL )
			{
				Message cMsg( ID_SAVE_LINK );
				cMsg.AddPointer( "view", m_pcView );
				cMsg.AddString( "url", url.string().utf8().data() );
				m_pcView->GetWindow()->PostMessage( &cMsg, m_pcView->GetWindow() );
			}
#endif
		}

	private:
		WebView* m_pcView;
};

/* The global mutex is supplied by WebCore */
extern Locker g_cGlobalMutex;

class WebFrameView;

/* XXXKV: There is a lot more that should be in here, including the implementation! */
class WebView::Private
{
	public:
		WebCore::Page* m_pcPage;
		WTF::RefPtr<WebCore::Frame> m_pcFrame;
		WTF::RefPtr<WebFrameView> m_pcFrameView;
		bool m_bStatusTextSet;
};

WebView::WebView( const Rect& cFrame, const String& cTitle, WebCore::ChromeClient* pcClient, uint32 nResizeMask, uint32 nFlags ) : View( cFrame, cTitle, nResizeMask, nFlags )
{
	m = new Private;
	m->m_bStatusTextSet = false;
	
	WebCore::EditorClientSyllable* pcEditor = new WebCore::EditorClientSyllable();
	WebViewContextMenuClient* pcContextMenu = new WebViewContextMenuClient( this );

	m->m_pcPage = new WebCore::Page(pcClient, pcContextMenu, pcEditor, new WebCore::DragClientSyllable(), new WebCore::InspectorClientSyllable() );
	pcEditor->setPage( m->m_pcPage );
	
	WebCore::Settings *settings = m->m_pcPage->settings();
	settings->setLoadsImagesAutomatically(true);
	settings->setMinimumFontSize(5);
	settings->setMinimumLogicalFontSize(5);
	settings->setShouldPrintBackgrounds(true);
	settings->setJavaScriptEnabled(true);

	settings->setDefaultFixedFontSize(14);
	settings->setDefaultFontSize(14);
	settings->setSerifFontFamily("Times New Roman");
	settings->setSansSerifFontFamily("Arial");
	settings->setFixedFontFamily("Courier");
	settings->setStandardFontFamily("Arial");
	
	WebViewFrameLoaderClient* pcLoaderClient = new WebViewFrameLoaderClient( this );

	m->m_pcFrame = new WebCore::Frame( m->m_pcPage, 0, pcLoaderClient );
	pcLoaderClient->setFrame( m->m_pcFrame.get() );
    
	m->m_pcFrameView = new WebFrameView( this, m->m_pcFrame.get() );
	m->m_pcFrameView->deref();
	m->m_pcFrame->setView(m->m_pcFrameView.get());
	m->m_pcFrame->init();
	m->m_pcFrameView->setParentWidget(this);
	m->m_pcFrame->view()->syllableWidget()->SetFrame( GetBounds() );
}

WebView::~WebView()
{
	g_cGlobalMutex.Lock();

	m->m_pcFrame->page()->focusController()->setActive( false );
	m->m_pcFrame->loader()->detachFromParent();

	/* XXXKV: Do these calls have side-effects? If not this entire block can be removed */
	WebCore::FrameView* pcFrameView = m->m_pcFrameView.get();
	WebCore::Frame* pcFrame = m->m_pcFrame.get();
	m->m_pcFrameView = NULL;
	m->m_pcFrame = NULL;

	delete( m->m_pcPage );	
	delete( m );

	g_cGlobalMutex.Unlock();
}

void WebView::KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers )
{
	if( nQualifiers & QUAL_CTRL )
	{
		switch( pzRawString[0] )
		{
			case VK_TAB:
			{
				GetWindow()->PostMessage( ID_ACTIVATE_URLEDIT, GetWindow() );
				break;
			}

			case 'r':
			{
				GetWindow()->PostMessage( ID_RELOAD, GetWindow() );
				break;
			}

			case 'f':
			{
				GetWindow()->PostMessage( ID_FIND, GetWindow() );
				break;
			}

			case VK_INSERT:
			case 'c':
			{
				GetWindow()->PostMessage( ID_COPY, GetWindow() );
				break;
			}

			case 'n':
			{
				GetWindow()->PostMessage( ID_NEW_WINDOW, GetWindow() );
				break;
			}

			default:
			{
				View::KeyDown( pzString, pzRawString, nQualifiers );
				break;
			}
		}
	}
	else
	{
		switch( pzRawString[0] )
		{
			case VK_FUNCTION_KEY:
			{
				Message *pcMsg = GetLooper()->GetCurrentMessage();
				ASSERT( pcMsg != NULL );
				int32 nKeyCode;

				if( pcMsg->FindInt32( "_raw_key", &nKeyCode ) != 0 )
					return;

				switch( nKeyCode )
				{
					case 4:	/* F3 */
					{
						GetWindow()->PostMessage( ID_REPEAT_FIND, GetWindow() );
						break;
					}

					case 6:	/* F5 */
					{
						GetWindow()->PostMessage( ID_RELOAD, GetWindow() );
						break;
					}

					default:
					{
						View::KeyDown( pzString, pzRawString, nQualifiers );
						break;
					}
				}
				break;
			}

			default:
			{
				View::KeyDown( pzString, pzRawString, nQualifiers );
				break;
			}
		}
	}
}

WebCore::Frame* WebView::GetWebCoreFrame()
{
	return m->m_pcFrame.get();
}

String WebView::GetCurrentURL()
{
	if( m->m_pcFrame->loader() == NULL || m->m_pcFrame->loader()->url().isEmpty() )
		return "";

	return String( m->m_pcFrame->loader()->url().string().utf8().data() );
}

void WebView::OpenURL( const WebCore::ResourceRequest& cRequest)
{
	if( cRequest.url().isEmpty() )
		return;

	printf( "Open URL %s\n", cRequest.url().string().utf8().data() );
	m->m_pcFrame->loader()->load( cRequest );
}

int WebView::GetBackListCount()
{
	return m->m_pcPage->backForwardList()->backListCount();
}

int WebView::GetForwardListCount()
{
	return m->m_pcPage->backForwardList()->forwardListCount();
}

void WebView::GoBack()
{
	m->m_pcPage->goBack();
}

void WebView::GoForward()
{
	m->m_pcPage->goForward();
}

void WebView::Reload()
{
	m->m_pcFrame->loader()->reload();
}

void WebView::Stop()
{
	m->m_pcFrame->loader()->stopAllLoaders();
}

bool WebView::IsLoading()
{
	return m->m_pcFrame->loader()->isLoading();
}

